package tutorialMiniCrm.client.tutorialMiniCrm.ui.desktop.outlines.pages;

import org.eclipse.scout.commons.annotations.Order;
import org.eclipse.scout.commons.exception.ProcessingException;
import org.eclipse.scout.rt.client.ui.action.menu.AbstractMenu;
import org.eclipse.scout.rt.client.ui.basic.cell.Cell;
import org.eclipse.scout.rt.client.ui.basic.table.AbstractTable;
import org.eclipse.scout.rt.client.ui.basic.table.ITableRow;
import org.eclipse.scout.rt.client.ui.basic.table.columns.AbstractDateColumn;
import org.eclipse.scout.rt.client.ui.basic.table.columns.AbstractDoubleColumn;
import org.eclipse.scout.rt.client.ui.basic.table.columns.AbstractIntegerColumn;
import org.eclipse.scout.rt.client.ui.basic.table.columns.AbstractSmartColumn;
import org.eclipse.scout.rt.client.ui.basic.table.columns.AbstractStringColumn;
import org.eclipse.scout.rt.client.ui.desktop.outline.pages.AbstractPageWithTable;
import org.eclipse.scout.rt.client.ui.desktop.outline.pages.IPage;
import org.eclipse.scout.rt.client.ui.desktop.outline.pages.ISearchForm;
import org.eclipse.scout.rt.client.ui.messagebox.MessageBox;
import org.eclipse.scout.rt.shared.AbstractIcons;
import org.eclipse.scout.rt.shared.services.common.code.ICodeType;
import org.eclipse.scout.rt.shared.services.common.jdbc.SearchFilter;
import org.eclipse.scout.rt.shared.services.lookup.LookupCall;
import org.eclipse.scout.service.SERVICES;

import tutorialMiniCrm.client.tutorialMiniCrm.ui.forms.PersonForm;
import tutorialMiniCrm.client.tutorialMiniCrm.ui.searchforms.PersonSearchForm;
import tutorialMiniCrm.shared.tutorialMiniCrm.Icons;
import tutorialMiniCrm.shared.tutorialMiniCrm.Texts;
import tutorialMiniCrm.shared.tutorialMiniCrm.services.code.SalutationCodeType;
import tutorialMiniCrm.shared.tutorialMiniCrm.services.lookup.CompanyLookupCall;
import tutorialMiniCrm.shared.tutorialMiniCrm.services.outline.IStandardOutlineService;
import tutorialMiniCrm.shared.tutorialMiniCrm.services.process.IPersonProcessService;
import tutorialMiniCrm.shared.tutorialMiniCrm.services.process.PersonSearchFormData;

public class PersonTablePage extends AbstractPageWithTable<PersonTablePage.Table>{

  @Override
  public String getConfiguredTitle(){
    return Texts.get("Personen");
  }

  @Override
  protected String getConfiguredIconId(){
    return AbstractIcons.Folder;
  }

  @Override
  protected Object[][] execLoadTableData(SearchFilter filter) throws ProcessingException{
    return SERVICES.getService(IStandardOutlineService.class).getPersonTableData((PersonSearchFormData)filter.getFormData());
  }

  @Override
  public Class<? extends ISearchForm> getConfiguredSearchForm(){
    return PersonSearchForm.class;
  }

  @Override
  public IPage execCreateChildPage(ITableRow row) throws ProcessingException {
    PersonNodePage childPage=new PersonNodePage();
    // TODO propagate the selected row to the childPage.
    return childPage;
  }

  @Order(10.0)
  public class Table extends AbstractTable{

    @Override
    protected String getConfiguredDefaultIconId(){
      return Icons.User2;
    }

    @Override
    protected void execDecorateRow(ITableRow row) throws ProcessingException{
      if(getTable().getSalutationColumn().getValue(row)!=null&&getTable().getSalutationColumn().getValue(row).equals(SalutationCodeType.FrauCode.ID)){
        row.setIconId(Icons.User3);
      }else{
        row.setIconId(Icons.User2);
      }
    }

    public PersonNrColumn getPersonNrColumn(){
      return getColumnSet().getColumnByClass(PersonNrColumn.class);
     }

    public OutlineColumn getOutlineColumn(){
      return getColumnSet().getColumnByClass(OutlineColumn.class);
     }

    public FirstNameColumn getFirstNameColumn(){
      return getColumnSet().getColumnByClass(FirstNameColumn.class);
     }

    public LastNameColumn getLastNameColumn(){
      return getColumnSet().getColumnByClass(LastNameColumn.class);
     }

    public BirthDateColumn getBirthDateColumn(){
      return getColumnSet().getColumnByClass(BirthDateColumn.class);
     }

    public IncomeColumn getIncomeColumn(){
      return getColumnSet().getColumnByClass(IncomeColumn.class);
     }

    public SalutationColumn getSalutationColumn(){
      return getColumnSet().getColumnByClass(SalutationColumn.class);
     }

    public DescriptionColumn getDescriptionColumn(){
      return getColumnSet().getColumnByClass(DescriptionColumn.class);
     }

    public CompanyColumn getCompanyColumn(){
      return getColumnSet().getColumnByClass(CompanyColumn.class);
     }

    @Order(10.0)
    public class PersonNrColumn extends AbstractIntegerColumn{

      @Override
      protected boolean getConfiguredVisible(){
        return false;
      }

      @Override
      protected boolean getConfiguredDisplayable(){
        return false;
      }
    }

    @Order(20.0)
    public class OutlineColumn extends AbstractStringColumn{
      @Override
      protected boolean getConfiguredVisible(){
        return false;
      }
    
      @Override
      protected boolean getConfiguredSummary(){
        return true;
      }
    
      @Override
      protected boolean getConfiguredDisplayable(){
        return false;
      }
    }

    @Order(30.0)
    public class FirstNameColumn extends AbstractStringColumn{

      @Override
      public String getConfiguredHeaderText(){
        return Texts.get("FirstName");
      }

      @Override
      protected int getConfiguredWidth(){
        return 100;
      }
    }

    @Order(40.0)
    public class LastNameColumn extends AbstractStringColumn{

      @Override
      protected String getConfiguredHeaderText(){
        return Texts.get("LastName");
      }

      @Override
      protected int getConfiguredWidth(){
        return 100;
      }
    
      @Override
      protected int getConfiguredSortIndex(){
        return 1;
      }
    
      @Override
      protected String getConfiguredBackgroundColor(){
        return "FFFF00";
      }
    }

    @Order(50.0)
    public class BirthDateColumn extends AbstractDateColumn{

      @Override
      public String getConfiguredHeaderText(){
        return Texts.get("Birthdate");
      }

      @Override
      protected int getConfiguredWidth(){
        return 100;
      }
    }

    @Order(60.0)
    public class IncomeColumn extends AbstractDoubleColumn{

      @Override
      public String getConfiguredHeaderText(){
        return Texts.get("Income");
      }

      @Override
      protected void execDecorateCell(Cell cell, ITableRow row) throws ProcessingException{
        if(getValue(row)!=null&&getValue(row)>1000){
          cell.setIconId(Icons.Check);
        }
        super.execDecorateCell(cell, row);
      }

      @Override
      protected int getConfiguredWidth(){
        return 100;
      }
    }

    @Order(70.0)
    public class SalutationColumn extends AbstractSmartColumn<Long>{

      @Override
      public String getConfiguredHeaderText(){
        return Texts.get("Salutation");
      }

      @Override
      protected int getConfiguredWidth(){
        return 120;
      }

      @Override
      protected Class<? extends ICodeType> getConfiguredCodeType(){
        return SalutationCodeType.class;
      }
    }

    @Order(80.0)
    public class DescriptionColumn extends AbstractStringColumn{

      @Override
      protected String getConfiguredHeaderText(){
        return Texts.get("Description");
      }

      @Override
      protected int getConfiguredWidth(){
        return 200;
      }
    }

    @Order(90.0)
    public class CompanyColumn extends AbstractSmartColumn<Long>{

      @Override
      public String getConfiguredHeaderText() {
        return Texts.get("Company");
      }

      @Override
      protected int getConfiguredWidth(){
        return 140;
      }

      @Override
      protected Class<? extends LookupCall> getConfiguredLookupCall(){
        return CompanyLookupCall.class;
      }
    }

    @Order(10.0)
    public class NewButtonMenu extends AbstractMenu{

      @Override
      public void execAction()  throws ProcessingException {
        PersonForm form = new PersonForm();
        form.startNew();
        
        form.waitFor();
        
        if(form.isFormStored()){
          reloadPage();
        }
      }

      @Override
      public String getConfiguredText() {
        return Texts.get("NewButton");
      }

      @Override
      protected boolean getConfiguredNonSelectionAction(){
        return true;
      }

      @Override
      protected boolean getConfiguredEmptySpaceAction(){
        return true;
      }
    }

    @Order(20.0)
    public class EditMenu extends AbstractMenu{

      @Override
      public void execAction()  throws ProcessingException {
        PersonForm form = new PersonForm();
        form.setPersonNr(getPersonNrColumn().getSelectedValue());
        form.startModify();
        form.waitFor();
        
        if(form.isFormStored()){
          reloadPage();
        }
      }

      @Override
      public String getConfiguredText() {
        return Texts.get("EditMenu");
      }
    }

    @Order(30.0)
    public class DeleteMenu extends AbstractMenu{
      @Override
      protected boolean getConfiguredMultiSelectionAction(){
        return true;
      }
      
      @Override
      protected void execAction() throws ProcessingException{
        if(MessageBox.showDeleteConfirmationMessage(getTable().getOutlineColumn().getSelectedDisplayTexts())){
          SERVICES.getService(IPersonProcessService.class).delete(getTable().getPersonNrColumn().getSelectedValues());

          reloadPage();
        }
      }

      @Override
      public String getConfiguredText() {
        return Texts.get("DeleteMenu");
      }
    }
  }
}
