package tutorialMiniCrm.client.tutorialMiniCrm.ui.forms;

import org.eclipse.scout.rt.client.ui.form.AbstractForm;
import org.eclipse.scout.commons.annotations.FormData;
import tutorialMiniCrm.shared.tutorialMiniCrm.Texts;
import org.eclipse.scout.commons.exception.ProcessingException;
import org.eclipse.scout.commons.exception.VetoException;
import org.eclipse.scout.rt.client.ui.form.fields.groupbox.AbstractGroupBox;
import org.eclipse.scout.commons.annotations.Order;
import java.lang.Long;
import org.eclipse.scout.rt.client.ui.form.AbstractFormHandler;

import tutorialMiniCrm.shared.tutorialMiniCrm.services.lookup.CompanyLookupCall;
import tutorialMiniCrm.shared.tutorialMiniCrm.services.process.ITipOfTheDayProcessService;
import tutorialMiniCrm.shared.tutorialMiniCrm.services.process.PersonFormData;
import org.eclipse.scout.service.SERVICES;
import tutorialMiniCrm.shared.tutorialMiniCrm.services.process.IPersonProcessService;
import tutorialMiniCrm.shared.tutorialMiniCrm.security.UpdatePersonPermission;
import org.eclipse.scout.rt.client.ui.form.fields.stringfield.AbstractStringField;
import tutorialMiniCrm.client.tutorialMiniCrm.ui.forms.PersonForm.MainBox.LastNameField;
import tutorialMiniCrm.client.tutorialMiniCrm.ui.forms.PersonForm.MainBox.FirstNameField;
import org.eclipse.scout.rt.client.ui.form.fields.datefield.AbstractDateField;
import tutorialMiniCrm.client.tutorialMiniCrm.ui.forms.PersonForm.MainBox.BirthDateField;
import org.eclipse.scout.rt.client.ui.form.fields.doublefield.AbstractDoubleField;
import tutorialMiniCrm.client.tutorialMiniCrm.ui.forms.PersonForm.MainBox.IncomeField;
import org.eclipse.scout.rt.client.ui.form.fields.smartfield.AbstractSmartField;
import tutorialMiniCrm.client.tutorialMiniCrm.ui.forms.PersonForm.MainBox.SalutationField;
import org.eclipse.scout.rt.client.ui.form.fields.button.AbstractOkButton;
import tutorialMiniCrm.client.tutorialMiniCrm.ui.forms.PersonForm.MainBox.OkButton;
import org.eclipse.scout.rt.client.ui.form.fields.button.AbstractCancelButton;
import org.eclipse.scout.rt.shared.services.lookup.LookupCall;

import tutorialMiniCrm.client.tutorialMiniCrm.ui.forms.PersonForm.MainBox.CancelButton;
import tutorialMiniCrm.client.tutorialMiniCrm.ui.forms.PersonForm.MainBox.CompanyField;
import org.eclipse.scout.rt.shared.services.common.code.ICodeType;
import tutorialMiniCrm.shared.tutorialMiniCrm.services.code.SalutationCodeType;
import tutorialMiniCrm.client.tutorialMiniCrm.ui.forms.PersonForm.MainBox.FirstGroup;
import tutorialMiniCrm.client.tutorialMiniCrm.ui.forms.PersonForm.MainBox.SecondGroup;
import tutorialMiniCrm.client.tutorialMiniCrm.ui.forms.PersonForm.MainBox.FirstGroup.DescriptionField;
import org.eclipse.scout.rt.client.ui.form.fields.htmlfield.AbstractHtmlField;
import tutorialMiniCrm.client.tutorialMiniCrm.ui.forms.PersonForm.MainBox.SecondGroup.TipOfTheDayField;
import tutorialMiniCrm.shared.tutorialMiniCrm.Icons;

@FormData
public class PersonForm extends AbstractForm{

  private Integer m_personNr;


  public PersonForm() throws ProcessingException{
    super();
  }

  @FormData
  public Integer getPersonNr() {
    return m_personNr;
  }

  @FormData
  public void setPersonNr(Integer personNr) {
    m_personNr = personNr;
  }

  @Override
  public String getConfiguredTitle() {
    return Texts.get("Form");
  }

  public MainBox getMainBox(){
      return getFieldByClass(MainBox.class);
  }

  public FirstNameField getFirstNameField(){
      return getFieldByClass(FirstNameField.class);
  }

  public LastNameField getLastNameField(){
      return getFieldByClass(LastNameField.class);
  }

  public BirthDateField getBirthDateField(){
      return getFieldByClass(BirthDateField.class);
  }

  public IncomeField getIncomeField(){
      return getFieldByClass(IncomeField.class);
  }

  public SalutationField getSalutationField(){
      return getFieldByClass(SalutationField.class);
  }

  public CompanyField getCompanyField(){
      return getFieldByClass(CompanyField.class);
  }

  public FirstGroup getFirstGroup(){
      return getFieldByClass(FirstGroup.class);
  }

  public DescriptionField getDescriptionField(){
      return getFieldByClass(DescriptionField.class);
  }

  public SecondGroup getSecondGroup(){
      return getFieldByClass(SecondGroup.class);
  }

  public TipOfTheDayField getTipOfTheDayField(){
      return getFieldByClass(TipOfTheDayField.class);
  }

  public OkButton getOkButton(){
      return getFieldByClass(OkButton.class);
  }

  public CancelButton getCancelButton(){
      return getFieldByClass(CancelButton.class);
  }

  public void startNew()throws ProcessingException {
    startInternal(new NewHandler());
  }

  public void startModify()throws ProcessingException {
    startInternal(new ModifyHandler());
  }

  @Order(10.0)
  public class MainBox extends AbstractGroupBox{

    @Override
    protected int getConfiguredGridColumnCount(){
      return 1;
    }

    @Order(10.0)
    public class FirstNameField extends AbstractStringField{

      @Override
      public String getConfiguredLabel(){
        return Texts.get("FirstName");
      }
    }

    @Order(20.0)
    public class LastNameField extends AbstractStringField{

      @Override
      public String getConfiguredLabel(){
        return Texts.get("LastName");
      }
    }

    @Order(30.0)
    public class BirthDateField extends AbstractDateField{

      @Override
      public String getConfiguredLabel(){
        return Texts.get("Birthdate");
      }
    }

    @Order(40.0)
    public class IncomeField extends AbstractDoubleField{

      @Override
      public String getConfiguredLabel(){
        return Texts.get("Income");
      }
      
      @Override
      protected Double execValidateValue(Double rawValue) throws ProcessingException{
        if(rawValue!=null&&rawValue<0){
          throw new VetoException(Texts.get("PleaseEnterNumberAboveZero"));
        } else {
          clearErrorStatus();
        }
        return super.execValidateValue(rawValue);
      }
    }

    @Order(50.0)
    public class SalutationField extends AbstractSmartField<Long>{

      @Override
      public String getConfiguredLabel(){
        return Texts.get("Salutation");
      }

      @Override
      protected Class<? extends ICodeType<?>> getConfiguredCodeType(){
        return SalutationCodeType.class;
      }
    }

    @Order(60.0)
    public class CompanyField extends AbstractSmartField<Long>{

      @Override
      public String getConfiguredLabel() {
        return Texts.get("Company");
      }

      @Override
      protected boolean getConfiguredMandatory(){
        return true;
      }

      @Override
      protected Class<? extends LookupCall> getConfiguredLookupCall(){
        return CompanyLookupCall.class;
      }
    }

    @Order(70.0)
    public class FirstGroup extends AbstractGroupBox{

      @Override
      protected String getConfiguredLabel(){
        return Texts.get("Groupbox1");
      }

      @Override
      protected String getConfiguredBorderDecoration(){
        return AbstractGroupBox.BORDER_DECORATION_SECTION;
      }

      @Order(10.0)
      public class DescriptionField extends AbstractStringField{

        @Override
        public String getConfiguredLabel() {
          return Texts.get("Description");
        }
      
        @Override
        protected boolean getConfiguredWrapText(){
          return true;
        }
      
        @Override
        protected boolean getConfiguredMultilineText(){
          return true;
        }
      
        @Override
        protected int getConfiguredGridH(){
          return 2;
        }
      }
    }

    @Order(80.0)
    public class SecondGroup extends AbstractGroupBox{

      @Override
      protected String getConfiguredLabel(){
        return Texts.get("Groupbox2");
      }

      @Override
      protected String getConfiguredBorderDecoration(){
        return AbstractGroupBox.BORDER_DECORATION_LINE;
      }

      @Order(10.0)
      public class TipOfTheDayField extends AbstractHtmlField{

        @Override
        public String getConfiguredLabel() {
          return Texts.get("TipOfTheDay");
        }
      
        @Override
        protected boolean getConfiguredLabelVisible(){
          return false;
        }
      
        @Override
        protected int getConfiguredGridH(){
          return 5;
        }
      }
    }

    @Order(90.0)
    public class OkButton extends AbstractOkButton{
      @Override
      protected String getConfiguredIconId(){
        return Icons.Check;
      }
    }

    @Order(100.0)
    public class CancelButton extends AbstractCancelButton{
      @Override
      protected String getConfiguredIconId(){
        return Icons.Delete2;
      }
    }
  }

  @Order(10.0)
  public class NewHandler extends AbstractFormHandler{

    @Override
    public void execLoad() throws ProcessingException{
      IPersonProcessService service = SERVICES.getService(IPersonProcessService.class);
      PersonFormData formData = new PersonFormData();
      exportFormData(formData);
      formData = service.prepareCreate(formData);
      importFormData(formData);
    }
    
    @Override
    protected void execPostLoad() throws ProcessingException{
      getSecondGroup().setView(false, false, false);
    }

    @Override
    public void execStore() throws ProcessingException{
      IPersonProcessService service = SERVICES.getService(IPersonProcessService.class);
      PersonFormData formData = new PersonFormData();
      exportFormData(formData);
      formData = service.create(formData);
    }
  }

  @Order(20.0)
  public class ModifyHandler extends AbstractFormHandler{

    @Override
    public void execLoad() throws ProcessingException{
      IPersonProcessService service = SERVICES.getService(IPersonProcessService.class);
      PersonFormData formData = new PersonFormData();
      exportFormData(formData);
      formData = service.load(formData);
      importFormData(formData);
      setEnabledPermission(new UpdatePersonPermission());
    }
    
    @Override
    protected void execPostLoad() throws ProcessingException{
      String tipOfTheDay=SERVICES.getService(ITipOfTheDayProcessService.class).getTipOfTheDay();
      getTipOfTheDayField().setValue("<html><h1>Tip Of The Day</h1>"+tipOfTheDay+"</html>");
    }

    @Override
    public void execStore() throws ProcessingException{
      IPersonProcessService service = SERVICES.getService(IPersonProcessService.class);
      PersonFormData formData = new PersonFormData();
      exportFormData(formData);
      formData = service.store(formData);
    }
  }
}
