package tutorialMiniCrm.client.tutorialMiniCrm.ui.searchforms;

import org.eclipse.scout.commons.annotations.FormData;
import org.eclipse.scout.commons.annotations.Order;
import org.eclipse.scout.commons.exception.ProcessingException;
import org.eclipse.scout.commons.exception.VetoException;
import org.eclipse.scout.rt.client.ui.desktop.outline.pages.AbstractSearchForm;
import org.eclipse.scout.rt.client.ui.form.AbstractFormHandler;
import org.eclipse.scout.rt.client.ui.form.fields.button.AbstractResetButton;
import org.eclipse.scout.rt.client.ui.form.fields.button.AbstractSearchButton;
import org.eclipse.scout.rt.client.ui.form.fields.datefield.AbstractDateField;
import org.eclipse.scout.rt.client.ui.form.fields.doublefield.AbstractDoubleField;
import org.eclipse.scout.rt.client.ui.form.fields.groupbox.AbstractGroupBox;
import org.eclipse.scout.rt.client.ui.form.fields.sequencebox.AbstractSequenceBox;
import org.eclipse.scout.rt.client.ui.form.fields.smartfield.AbstractSmartField;
import org.eclipse.scout.rt.client.ui.form.fields.stringfield.AbstractStringField;
import org.eclipse.scout.rt.client.ui.form.fields.tabbox.AbstractTabBox;
import org.eclipse.scout.rt.shared.services.common.jdbc.SearchFilter;

import tutorialMiniCrm.client.tutorialMiniCrm.ui.searchforms.PersonSearchForm.MainBox.ResetButton;
import tutorialMiniCrm.client.tutorialMiniCrm.ui.searchforms.PersonSearchForm.MainBox.SearchButton;
import tutorialMiniCrm.client.tutorialMiniCrm.ui.searchforms.PersonSearchForm.MainBox.TabBox;
import tutorialMiniCrm.client.tutorialMiniCrm.ui.searchforms.PersonSearchForm.MainBox.TabBox.FieldGroup;
import tutorialMiniCrm.client.tutorialMiniCrm.ui.searchforms.PersonSearchForm.MainBox.TabBox.FieldGroup.BirthDateBox;
import tutorialMiniCrm.client.tutorialMiniCrm.ui.searchforms.PersonSearchForm.MainBox.TabBox.FieldGroup.FirstNameField;
import tutorialMiniCrm.client.tutorialMiniCrm.ui.searchforms.PersonSearchForm.MainBox.TabBox.FieldGroup.LastNameField;
import tutorialMiniCrm.client.tutorialMiniCrm.ui.searchforms.PersonSearchForm.MainBox.TabBox.FieldGroup.SalutationField;
import tutorialMiniCrm.client.tutorialMiniCrm.ui.searchforms.PersonSearchForm.MainBox.TabBox.FieldGroup.SequenceBox;
import tutorialMiniCrm.client.tutorialMiniCrm.ui.searchforms.PersonSearchForm.MainBox.TabBox.FieldGroup.SequenceBox.SequenceFrom;
import tutorialMiniCrm.client.tutorialMiniCrm.ui.searchforms.PersonSearchForm.MainBox.TabBox.FieldGroup.SequenceBox.SequenceTo;
import tutorialMiniCrm.shared.tutorialMiniCrm.Texts;
import tutorialMiniCrm.shared.tutorialMiniCrm.services.process.PersonSearchFormData;
import org.eclipse.scout.rt.shared.services.common.code.ICodeType;
import tutorialMiniCrm.shared.tutorialMiniCrm.services.code.SalutationCodeType;
import org.eclipse.scout.rt.shared.AbstractIcons;

@FormData
public class PersonSearchForm extends AbstractSearchForm{

  public PersonSearchForm() throws ProcessingException{
    super();
  }

  @Override
  protected void execResetSearchFilter(SearchFilter searchFilter) throws ProcessingException{
    super.execResetSearchFilter(searchFilter);
    PersonSearchFormData formData = new PersonSearchFormData();
    exportFormData(formData);
    searchFilter.setFormData(formData);
  }

  @Override
  public void startSearch()throws ProcessingException {
    startInternal(new SearchHandler());
  }

  public MainBox getMainBox(){
      return getFieldByClass(MainBox.class);
  }

  public TabBox getTabBox(){
      return getFieldByClass(TabBox.class);
  }

  public FieldGroup getFieldGroup(){
      return getFieldByClass(FieldGroup.class);
  }

  public FirstNameField getFirstNameField(){
      return getFieldByClass(FirstNameField.class);
  }

  public LastNameField getLastNameField(){
      return getFieldByClass(LastNameField.class);
  }

  public BirthDateBox getBirthDateBox(){
      return getFieldByClass(BirthDateBox.class);
  }

  public SalutationField getSalutationField(){
      return getFieldByClass(SalutationField.class);
  }

  public SequenceBox getSequenceBox(){
      return getFieldByClass(SequenceBox.class);
  }

  public SequenceFrom getSequenceFrom(){
      return getFieldByClass(SequenceFrom.class);
  }

  public SequenceTo getSequenceTo(){
      return getFieldByClass(SequenceTo.class);
  }

  public ResetButton getResetButton(){
      return getFieldByClass(ResetButton.class);
  }

  public SearchButton getSearchButton(){
      return getFieldByClass(SearchButton.class);
  }

  @Order(10.0)
  public class MainBox extends AbstractGroupBox{

    @Order(10.0)
    public class TabBox extends AbstractTabBox{

      @Order(10.0)
      public class FieldGroup extends AbstractGroupBox{

        @Order(10.0)
        public class FirstNameField extends AbstractStringField{

          @Override
          public String getConfiguredLabel(){
            return Texts.get("FirstName");
          }
        }

        @Order(20.0)
        public class LastNameField extends AbstractStringField{

          @Override
          public String getConfiguredLabel(){
            return Texts.get("LastName");
          }
        }

        @Order(30.0)
        public class BirthDateBox extends AbstractSequenceBox{

          @Override
          public String getConfiguredLabel(){
            return Texts.get("Birthdate");
          }
          
          @Order(10.0)
          public class BirthDateBoxFrom extends AbstractDateField{

            @Override
            public String getConfiguredLabel() {
              return Texts.get("from");
            }
          }

          @Order(20.0)
          public class BirthDateBoxTo extends AbstractDateField{

            @Override
            public String getConfiguredLabel() {
              return Texts.get("to");
            }
          }
        }

        @Order(50.0)
        public class SalutationField extends AbstractSmartField<Long>{

          @Override
          public String getConfiguredLabel(){
            return Texts.get("Salutation");
          }
        
          @Override
          protected Class<? extends ICodeType<?>> getConfiguredCodeType(){
            return SalutationCodeType.class;
          }
        }

        @Order(60.0)
        public class SequenceBox extends AbstractSequenceBox{

          @Override
          public String getConfiguredLabel(){
            return Texts.get("Income");
          }

          @Order(10.0)
          public class SequenceFrom extends AbstractDoubleField{

            @Override
            public String getConfiguredLabel() {
              return Texts.get("from");
            }
            
            @Override
            protected Double execValidateValue(Double rawValue) throws ProcessingException{
              if(rawValue!=null&&rawValue<0.0){
                throw new VetoException("Bitte einen Wert grsser als 0 eingeben");
              }else{
                clearErrorStatus();
              }
              return super.execValidateValue(rawValue);
            }
          }

          @Order(20.0)
          public class SequenceTo extends AbstractDoubleField{

            @Override
            public String getConfiguredLabel() {
              return Texts.get("to");
            }
            
            @Override
            protected Double execValidateValue(Double rawValue) throws ProcessingException{
              if(rawValue!=null&&rawValue<0.0){
                throw new VetoException("Bitte einen Wert grsser als 0 eingeben");
              }else{
                clearErrorStatus();
              }
              return super.execValidateValue(rawValue);
            }
          }
        }
      
        @Override
        protected String getConfiguredLabel(){
          return Texts.get("Suchbegriffe");
        }
      }
    }

    @Order(20.0)
    public class ResetButton extends AbstractResetButton{
    }

    @Order(30.0)
    public class SearchButton extends AbstractSearchButton{
      @Override
      protected String getConfiguredIconId(){
        return AbstractIcons.SmartFieldBrowse;
      }
    }
  }

  @Order(10.0)
  public class SearchHandler extends AbstractFormHandler{

    @Override
    public void execLoad(){
      // TODO Auto-generated method stub
    }
  }

  @Override
  protected String getConfiguredTitle(){
    return Texts.get("SuchePersonen");
  }
}
