package tutorialMiniCrm.server.tutorialMiniCrm.services.process;

import org.eclipse.scout.commons.exception.ProcessingException;
import org.eclipse.scout.commons.exception.VetoException;
import org.eclipse.scout.commons.holders.IntegerHolder;
import org.eclipse.scout.commons.holders.NVPair;
import org.eclipse.scout.rt.server.services.common.jdbc.SQL;
import org.eclipse.scout.rt.shared.services.common.security.ACCESS;
import org.eclipse.scout.service.AbstractService;

import tutorialMiniCrm.shared.tutorialMiniCrm.Texts;
import tutorialMiniCrm.shared.tutorialMiniCrm.security.CreatePersonPermission;
import tutorialMiniCrm.shared.tutorialMiniCrm.security.ReadPersonPermission;
import tutorialMiniCrm.shared.tutorialMiniCrm.security.UpdatePersonPermission;
import tutorialMiniCrm.shared.tutorialMiniCrm.services.process.IPersonProcessService;
import tutorialMiniCrm.shared.tutorialMiniCrm.services.process.PersonFormData;

public class PersonProcessService extends AbstractService implements IPersonProcessService{

  public PersonFormData prepareCreate(PersonFormData formData) throws ProcessingException{
    if(!ACCESS.check(new CreatePersonPermission())){
      throw new VetoException(Texts.get("AuthorizationFailed"));
    }
    // TODO business logic here
    return formData;
  }

  public PersonFormData create(PersonFormData formData) throws ProcessingException{
    if(!ACCESS.check(new CreatePersonPermission())){
      throw new VetoException(Texts.get("AuthorizationFailed"));
    }
    IntegerHolder personNr = new IntegerHolder();
    SQL.selectInto("SELECT COUNT(*) + 10 FROM PERSON INTO :personNr", new NVPair("personNr", personNr));
    formData.setPersonNr(personNr.getValue());
    
    SQL.insert("INSERT INTO PERSON (" +
                                		"PERSON_NR, " +
                                		"FIRST_NAME, " +
                                		"LAST_NAME, " +
                                		"EVT_BIRTH, " +
                                		"SALUTATION_UID, " +
                                		"INCOME, " +
                                		"COMPANY_NR, " +
                                		"DESCRIPTION ) " +
                                		"VALUES (" +
                                		":personNr, " +
                                		":firstName, " +
                                		":lastName, " +
                                		":birthDate, " +
                                		":salutation, " +
                                		":income, " +
                                		":company, " +
                                		":description)", formData);
    
    return formData;
  }

  public PersonFormData load(PersonFormData formData) throws ProcessingException{
    if(!ACCESS.check(new ReadPersonPermission())){
      throw new VetoException(Texts.get("AuthorizationFailed"));
    }
    
    SQL.selectInto(
        "SELECT FIRST_NAME, " +
    		"       LAST_NAME, " +
    		"       INCOME, " +
    		"       SALUTATION_UID, " +
    		"       EVT_BIRTH, " +
    		"       COMPANY_NR," +
    		"       DESCRIPTION " +
    		"FROM   PERSON " +
    		"WHERE  PERSON_NR = :personNr " +
    		"INTO   :firstName, " +
    		"       :lastName, " +
    		"       :income, " +
    		"       :salutation, " +
    		"       :birthDate, " +
    		"       :company, " +
    		"       :description ", formData);
    
    return formData;
  }

  public PersonFormData store(PersonFormData formData) throws ProcessingException{
    if(!ACCESS.check(new UpdatePersonPermission())){
      throw new VetoException(Texts.get("AuthorizationFailed"));
    }
    
    SQL.update(
        "UPDATE PERSON SET" +
        "       FIRST_NAME = :firstName, " +
        "       LAST_NAME = :lastName, " +
        "       INCOME = :income, " +
        "       SALUTATION_UID = :salutation, " +
        "       EVT_BIRTH = :birthDate, " +
        "       DESCRIPTION = :description " +
        "WHERE  PERSON_NR = :personNr", formData);
    
    return formData;
  }

  public void delete(Integer[] personNr) throws ProcessingException{
    SQL.delete("DELETE FROM PERSON WHERE PERSON_NR = :personNr", new NVPair("personNr", personNr));
  }
}
